@@ -31,9 +31,10 @@
 #' @param linkage character: heirarchical clustering linkage method - see ?hclust
 #' @param mzdec integer: number of decimal places used in printing m/z values
 #' @param cor.method character: which correlational method used to calculate 'r' - see ?cor
+#' @param rt.only.low.n logical: default = TRUE  At low injection numbers, correlational relationships of peak intensities may be unreliable.  by defualt ramclustR will simply ignore the correlational r value and cluster on retention time alone.  if you wish to use correlation with at n < 5, set this value to FALSE.
 #' @param fftempdir valid path: if there are file size limitations on the default ff pacakge temp directory  - getOptions('fftempdir') - you can change the directory used as the fftempdir with this option.
 #'
-#' @details Main clustering function output - see citation for algorithm description of vignette('RAMClustR') for a walk through.  batch.qc. normalization requires input of three vectors (1) batch (2) order (3) qc.   This is a feature centric normalization approach which adjusts signal intensities first by comparing batch median intensity of each feature (one feature at a time) QC signal intensity to full dataset median to correct for systematic batch effects and then secondly to apply a local QC median vs global median sample correction to correct for run order effects.
+#' @details Main clustering function output - see citation for algorithm description or vignette('RAMClustR') for a walk through.  batch.qc. normalization requires input of three vectors (1) batch (2) order (3) qc.   This is a feature centric normalization approach which adjusts signal intensities first by comparing batch median intensity of each feature (one feature at a time) QC signal intensity to full dataset median to correct for systematic batch effects and then secondly to apply a local QC median vs global median sample correction to correct for run order effects.
 #' @return   $featclus: integer vector of cluster membership for each feature
 #' @return   $frt: feature retention time, in whatever units were fed in (xcms uses seconds, by default)
 #' @return   $fmz: feature retention time, reported in number of decimal points selected in ramclustR function
@@ -100,6 +101,7 @@ ramclustR<- function(  xcmsObj=NULL,
                        linkage="average",
                        mzdec=3,
                        cor.method="pearson",
+                       rt.only.low.n = TRUE,
                        fftempdir = NULL
 ) {
   
@@ -268,16 +270,20 @@ ramclustR<- function(  xcmsObj=NULL,
           times[do[x]]<-  as.numeric((xcmsObj@groups[do[x],"rtmin"]+ xcmsObj@groups[do[x],"rtmax"])/2)
         }
       }
+      
+      if (normalize == "batch.qc") {
+        qc2 <- qc[msfiles]
+        qc <- qc[msfiles]
+        order2 <- order[msfiles]
+        order <- order[msfiles]
+        batch2 <- batch[msfiles]
+        batch <- batch[msfiles]
+      }
+      
       #if(any(is.na(times))) {stop("na values still present")} else {print("NAs removed")}
     }
   }
   
-  if(nrow(data1) < 5) {
-    warning('\n', "too few samples to use correlational similarity, clustering by retention time only", '\n')
-    ramclustObj$history <- paste(ramclustObj$history,
-                                 "Since there were fewer than five injections, clustering was performed only using retention time simiilarity.")
-  }
-  
   ## if using batch.qc check for proper information
   if(normalize == "batch.qc") {
     if(!all.equal(length(batch), length(qc), length(order), nrow(data1))) {
@@ -374,29 +380,39 @@ ramclustR<- function(  xcmsObj=NULL,
                      " bactches, and normization was based on ", length(which(qc)), " recognized QC samples.", sep = "") 
     
     pdf(file = "norm.plots.pdf", height = 4, width = 9)
-    
+    max.ratio <- 4
     for(z in 1:ncol(data1)) {
       # z <- sample(1:ncol(data1), 1)
-      tmp <- data1[,z]
+      tmp <- data1[,z]  
       featmed <- mean(tmp[qc])
       tmpn <- tmp
       
-      for( i in 1:max(batch)) {
+      for( i in unique(batch)) {
         
         do <- which (batch == i)
         doqc <- which(batch == i & qc)
-        names(doqc) <- names(tmp[doqc])
+        # names(doqc) <- names(tmp[doqc])
         
         ## use only 'typical' QC sample values from the given batch
         ## outliers are detected using the standard boxplot definition (1.5 * the interquartile range)
-        out <- boxplot(tmp[doqc], plot = FALSE)$out
-        if(length(out)>0) doqc <- doqc[!(names(doqc) %in% names(out))]
+        # out <- boxplot(tmp[doqc], plot = FALSE)$out
+        sds <- 1.96
+        lcl <- mean(tmp[doqc]) - (sds*sd(tmp[doqc]))
+        ucl <- mean(tmp[doqc]) + (sds*sd(tmp[doqc]))
+        keep <- which(tmp[doqc] > lcl & tmp[doqc] < ucl)
+        #if(length(out)>0) doqc <- doqc[!(names(doqc) %in% names(out))]
+        if(length(keep)>0) doqc <- doqc[keep]
         
         batchmed <- mean(tmpn[doqc])
         f <- batchmed / featmed
-        if(abs(log2(f)) > 4)  {
-          if(f > 1) {f <- 4}
-          if(f < 1) {f <- 0.25}
+        # cat("i: ", i, '\n')
+        # cat("f: ", f, '\n')
+        # cat("batchmed: ", batchmed, '\n')
+        # cat("featmed: ", featmed, '\n')
+        if(is.na(f)) next
+        if(abs(log2(f)) > max.ratio)  {
+          if(f > 1) {f <- max.ratio}
+          if(f < 1) {f <- 1/max.ratio}
         }
         tmpn[do] <- tmp[do]/f
         
@@ -431,12 +447,14 @@ ramclustR<- function(  xcmsObj=NULL,
           }
           if(is.na(localmed)) {localmed <- batchmed}
           f <- localmed/batchmed
+          if(is.na(f)) next
           
-          if(abs(log2(f)) > 4) {
+          if(abs(log2(f)) > max.ratio) {
             # f <- batchmed / featmed
-            if(f > 1) {f <- 4}
-            if(f < 1) {f <- 0.25}
+            if(f > 1) {f <- max.ratio}
+            if(f < 1) {f <- 1/max.ratio}
           }
+          
           tmpn[x] <- tmpnqc[x] / f
           rm(localmed); rm(f)
         }
@@ -454,7 +472,7 @@ ramclustR<- function(  xcmsObj=NULL,
       
     }
     
-    if(mslev > 1) {
+    if(mslev == 2) {
       qc    <- qc2
       batch <- batch2
       order <- order2
@@ -464,7 +482,7 @@ ramclustR<- function(  xcmsObj=NULL,
         featmed <- mean(tmp[qc])
         tmpn <- tmp
         
-        for( i in 1:max(batch)) {
+        for( i in unique(batch)) {
           
           do <- which (batch == i)
           doqc <- which(batch == i & qc)
@@ -472,14 +490,19 @@ ramclustR<- function(  xcmsObj=NULL,
           
           ## use only 'typical' QC sample values from the given batch
           ## outliers are detected using the standard boxplot definition (1.5 * the interquartile range)
-          out <- boxplot(tmp[doqc], plot = FALSE)$out
-          if(length(out)>0) doqc <- doqc[!(names(doqc) %in% names(out))]
+          #out <- boxplot(tmp[doqc], plot = FALSE)$out
+          #if(length(out)>0) doqc <- doqc[!(names(doqc) %in% names(out))]
+          lcl <- mean(tmp[doqc]) - (sds*sd(tmp[doqc]))
+          ucl <- mean(tmp[doqc]) + (sds*sd(tmp[doqc]))
+          keep <- which(tmp[doqc] > lcl & tmp[doqc] < ucl)
+          if(length(keep)>0) doqc <- doqc[keep]
           
           batchmed <- mean(tmpn[doqc])
           f <- batchmed / featmed
-          if(abs(log2(f)) > 4)  {
-            if(f > 1) {f <- 4}
-            if(f < 1) {f <- 0.25}
+          if(is.na(f)) next
+          if(abs(log2(f)) > max.ratio)  {
+            if(f > 1) {f <- max.ratio}
+            if(f < 1) {f <- 1/max.ratio}
           }
           tmpn[do] <- tmp[do]/f
           
@@ -514,11 +537,11 @@ ramclustR<- function(  xcmsObj=NULL,
             }
             if(is.na(localmed)) {localmed <- batchmed}
             f <- localmed/batchmed
-            
-            if(abs(log2(f)) > 4) {
+            if(is.na(f)) next
+            if(abs(log2(f)) > max.ratio) {
               # f <- batchmed / featmed
-              if(f > 1) {f <- 4}
-              if(f < 1) {f <- 0.25}
+              if(f > 1) {f <- max.ratio}
+              if(f < 1) {f <- 1/max.ratio}
             }
             tmpn[x] <- tmpnqc[x] / f
             rm(localmed); rm(f)
@@ -564,7 +587,7 @@ ramclustR<- function(  xcmsObj=NULL,
   ########
   # set off ff matrix system for holding data. 
   # manages RAM demands a bit.  
-  ffmat<-ff(vmode="double", dim=c(n, n), initdata = 0) ##reset to 1 if necessary
+  ffmat<-ff::ff(vmode="double", dim=c(n, n), initdata = 0) ##reset to 1 if necessary
   gc()
   #Sys.sleep((n^2)/10000000)
   #gc()
@@ -598,14 +621,14 @@ ramclustR<- function(  xcmsObj=NULL,
                      
                      digits=20 )
         
-        if(nrow(data1) >= 5) {
-          temp2<-round (exp(-((1-(pmax(  cor(data1[,startr:stopr], data1[,startc:stopc], method=cor.method),
-                                       cor(data1[,startr:stopr], data2[,startc:stopc], method=cor.method),
-                                       cor(data2[,startr:stopr], data2[,startc:stopc], method=cor.method)  )))^2)/(2*(sr^2))), 
-                      
-                      digits=20 )	
+        if(nrow(data1) < 5 & rt.only.low.n) {
+          temp2 <- matrix(data = 1, nrow = length(startr:stopr), ncol = length(startc:stopc))
         } else {
-          tmp2 <- matrix(data = 1, nrow = length(startr:stopr), ncol = length(startc:stopc))
+          temp2<-round (exp(-((1-(pmax(  cor(data1[,startr:stopr], data1[,startc:stopc], method=cor.method),
+                                         cor(data1[,startr:stopr], data2[,startc:stopc], method=cor.method),
+                                         cor(data2[,startr:stopr], data2[,startc:stopc], method=cor.method)  )))^2)/(2*(sr^2))), 
+                        
+                        digits=20 )	
         }
         #ffcor[startr:stopr, startc:stopc]<-temp
         temp<- 1-(temp1*temp2)
@@ -671,7 +694,7 @@ ramclustR<- function(  xcmsObj=NULL,
   
   ########
   # cleanup
-  delete.ff(ffmat)
+  ff::delete.ff(ffmat)
   rm(ffmat)
   gc()
   if(!is.null(fftempdir)) {
@@ -681,7 +704,7 @@ ramclustR<- function(  xcmsObj=NULL,
   
   ########
   # cluster using fastcluster package,
-  system.time(ramclustObj<-hclust(ramclustObj, method=linkage))
+  system.time(ramclustObj<-fastcluster::hclust(ramclustObj, method=linkage))
   history <- paste(history,
                    "The feature similarity matrix was clustered using fastcluster package heirarchical clustering method using the",
                    linkage, "method."
@@ -691,12 +714,12 @@ ramclustR<- function(  xcmsObj=NULL,
   d<-Sys.time()    
   cat("fastcluster based clustering complete", '\n')
   if(minModuleSize==1) {
-    clus<-cutreeDynamicTree(ramclustObj, maxTreeHeight=hmax, deepSplit=deepSplit, minModuleSize=2)
+    clus<-dynamicTreeCut::cutreeDynamicTree(ramclustObj, maxTreeHeight=hmax, deepSplit=deepSplit, minModuleSize=2)
     sing<-which(clus==0)
     clus[sing]<-max(clus)+1:length(sing)
   }
   if(minModuleSize>1) {
-    clus<-cutreeDynamicTree(ramclustObj, maxTreeHeight=hmax, deepSplit=deepSplit, minModuleSize=minModuleSize)
+    clus<-dynamicTreeCut::cutreeDynamicTree(ramclustObj, maxTreeHeight=hmax, deepSplit=deepSplit, minModuleSize=minModuleSize)
   }
   gc()
   
@@ -750,7 +773,7 @@ ramclustR<- function(  xcmsObj=NULL,
   ramclustObj$cmpd<-paste("C", formatC(1:length(ramclustObj$clrt), digits = strl, flag = 0 ) , sep="")
   # cat(ramclustObj$cmpd[1:10], '\n')
   ramclustObj$ann<-ramclustObj$cmpd
-  ramclustObj$annconf<-rep("", length(ramclustObj$clrt))
+  ramclustObj$annconf<-rep(4, length(ramclustObj$clrt))
   ramclustObj$annnotes<-rep("", length(ramclustObj$clrt))
   ramclustObj$MSdata_unnormalized <- data1raw
   if(mslev == 2) {
@@ -852,6 +875,11 @@ ramclustR<- function(  xcmsObj=NULL,
     cat(paste("msp file complete", '\n')) 
   }  
   ramclustObj$history <- history
+  if(nrow(ramclustObj$MSdata) < 5 & rt.only.low.n) {
+    warning('\n', "too few samples to use correlational similarity, clustering by retention time only", '\n')
+    ramclustObj$history <- paste(ramclustObj$history,
+                                 "Since there were fewer than five injections, clustering was performed only using retention time simiilarity.")
+  }
   return(ramclustObj)
 }
 
